@@ -1,25 +1,11 @@
 /*
- * "$Id: htmldoc.h 1421 2005-08-07 11:51:33Z mike $"
+ * Header file for HTMLDOC, a HTML document processing program.
  *
- *   Header file for HTMLDOC, a HTML document processing program.
+ * Copyright 2011-2020 by Michael R Sweet.
+ * Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   Copyright 1997-2005 by Easy Software Products.
- *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3142 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
+ * This program is free software.  Distribution and use rights are outlined in
+ * the file "COPYING".
  */
 
 /*
@@ -162,7 +148,7 @@ VAR int		PDFPageMode	VALUE(PDF_OUTLINE),
 		PDFFirstPage	VALUE(PDF_CHAPTER_1),
 						/* First page */
 		PDFEffect	VALUE(PDF_NONE);/* Page transition effect */
-VAR float	PDFEffectDuration VALUE(1.0),	/* Page effect duration */
+VAR double	PDFEffectDuration VALUE(1.0),	/* Page effect duration */
 		PDFPageDuration	VALUE(10.0);	/* Page duration */
 VAR int		Encryption	VALUE(0),	/* Encrypt the PDF file? */
 		Permissions	VALUE(-4);	/* File permissions? */
@@ -188,7 +174,7 @@ VAR typeface_t	HeadFootType	VALUE(TYPE_HELVETICA);
 						/* Typeface for header & footer */
 VAR style_t	HeadFootStyle	VALUE(STYLE_NORMAL);
 						/* Type style */
-VAR float	HeadFootSize	VALUE(11.0f);	/* Size of header & footer */
+VAR double	HeadFootSize	VALUE(11.0f);	/* Size of header & footer */
 
 VAR char	*Header[3]	NULL3,		/* Header for regular pages */
 		*Header1[3]	NULL3,		/* Header for first pages */
@@ -199,6 +185,7 @@ VAR char	*Header[3]	NULL3,		/* Header for regular pages */
 						/* TOC title string */
 
 VAR char	TitleImage[1024] VALUE(""),	/* Title page image */
+		Letterhead[1024] VALUE(""),	/* Letterhead image */
 		LogoImage[1024]	VALUE(""),	/* Logo image */
 		BodyColor[255]	VALUE(""),	/* Body color */
 		BodyImage[1024]	VALUE(""),	/* Body image */
@@ -241,11 +228,14 @@ VAR const char	*PDFEffects[17]			/* Effect strings */
 VAR GUI		*BookGUI	VALUE(NULL);	/* GUI for book files */
 #  ifdef WIN32					/* Editor for HTML files */
 VAR char	HTMLEditor[1024] VALUE("notepad.exe \"%s\"");
+#  elif defined(__APPLE__)
+VAR char	HTMLEditor[1024] VALUE("bbedit %s");
+#  elif defined(__linux)
+VAR char	HTMLEditor[1024] VALUE("gedit %s");
 #  else
 VAR char	HTMLEditor[1024] VALUE("nedit %s");
 #  endif /* WIN32 */
-VAR int		Tooltips	VALUE(1);	/* Show tooltips? */
-VAR int		ModernSkin	VALUE(1);	/* Show modern skins? */
+VAR int		Tooltips	VALUE(0);	/* Show tooltips? */
 #endif /* HAVE_LIBFLTK */
 
 
@@ -255,6 +245,8 @@ VAR int		ModernSkin	VALUE(1);	/* Show modern skins? */
 
 extern int	pspdf_export(tree_t *document, tree_t *toc);
 
+extern int	epub_export(tree_t *document, tree_t *toc);
+
 extern int	html_export(tree_t *document, tree_t *toc);
 
 extern int	htmlsep_export(tree_t *document, tree_t *toc);
@@ -276,7 +268,3 @@ extern char	*format_number(int n, char f);
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
-
-/*
- * End of "$Id: htmldoc.h 1421 2005-08-07 11:51:33Z mike $".
- */
