@@ -1,7 +1,7 @@
 /*
  * HTML parsing routines for HTMLDOC, a HTML document processing program.
  *
- * Copyright 2011-2019 by Michael R Sweet.
+ * Copyright 2011-2020 by Michael R Sweet.
  * Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
  * This program is free software.  Distribution and use rights are outlined in
@@ -13,6 +13,7 @@
  */
 
 #include "htmldoc.h"
+#include "http.h"
 #include <ctype.h>
 
 
@@ -140,7 +141,7 @@ typeface_t	_htmlBodyFont = TYPE_TIMES,
 int		_htmlInitialized = 0;	/* Initialized glyphs yet? */
 char		_htmlCharSet[256] = "iso-8859-1";
 					/* Character set name */
-extern int	_htmlWidthsLoaded[TYPE_MAX][STYLE_MAX] =
+int		_htmlWidthsLoaded[TYPE_MAX][STYLE_MAX] =
 		{			/* Have the widths been loaded? */
 		  { 0, 0, 0, 0 },
 		  { 0, 0, 0, 0 },
@@ -1314,7 +1315,7 @@ htmlReadFile(tree_t     *parent,	// I - Parent tree entry
         	t->typeface = TYPE_SANS_SERIF;
 		break;
 	      }
-              else if (!strcasecmp(font, "mono"))
+              else if (!strcasecmp(font, "mono") || !strcasecmp(font, "monospace"))
 	      {
         	t->typeface = TYPE_MONOSPACE;
 		break;
@@ -1870,6 +1871,40 @@ htmlInsertTree(tree_t   *parent,/* I - Parent entry */
 }
 
 
+/*
+ * 'htmlMapUnicode()' - Map a Unicode character to the custom character set.
+ */
+
+uchar				/* O - Charset character */
+htmlMapUnicode(int ch)		/* I - Unicode character */
+{
+  uchar	newch;			/* New charset character */
+
+
+  // If we already have a mapping for this character, return it...
+  if (_htmlCharacters[ch])
+    return (_htmlCharacters[ch]);
+
+  if (_htmlUTF8 >= 0x100)
+  {
+    progress_error(HD_ERROR_READ_ERROR, "Too many Unicode code points.");
+    return (0);
+  }
+
+  newch = _htmlUTF8++;
+
+  _htmlCharacters[ch] = (uchar)newch;
+  _htmlUnicode[newch] = ch;
+  _htmlGlyphs[newch]  = _htmlGlyphsAll[ch];
+
+  for (int i = 0; i < TYPE_MAX; i ++)
+    for (int j = 0; j < STYLE_MAX; j ++)
+      _htmlWidths[i][j][newch] = _htmlWidthsAll[i][j][ch];
+
+  return (newch);
+}
+
+
 /*
  * 'htmlNewTree()' - Create a new tree node for the parent.
  */
@@ -3476,60 +3511,82 @@ fix_filename(char *filename,		/* I - Original filename */
 
   if (strncmp(base, "http://", 7) == 0 || strncmp(base, "https://", 8) == 0)
   {
-    strlcpy(newfilename, base, sizeof(newfilename));
-    base = strchr(newfilename, ':') + 3;
+    // Base is a URL...
+    char	scheme[32],		// URI scheme
+		userpass[256],		// Username:password
+		host[256],		// Hostname or IP address
+		resource[256];		// Resource path
+    int		port;			// Port number
+
+    httpSeparateURI(HTTP_URI_CODING_ALL, base, scheme, sizeof(scheme), userpass, sizeof(userpass), host, sizeof(host), &port, resource, sizeof(resource));
 
     if (filename[0] == '/')
     {
-      if ((slash = strchr(base, '/')) != NULL)
-        strlcpy(slash, filename, sizeof(newfilename) - (size_t)(slash - newfilename));
-      else
-        strlcat(newfilename, filename, sizeof(newfilename));
+      // Absolute path, so just use the server...
+      httpAssembleURI(HTTP_URI_CODING_ALL, newfilename, sizeof(newfilename), scheme, userpass, host, port, filename);
+    }
+    else
+    {
+      // Relative path, strip the last component from the resource...
+      if ((slash = strrchr(resource, '/')) != NULL)
+	*slash = '\0';
 
-      return (newfilename);
+      // Handle "../" in filename...
+      while (!strncmp(filename, "../", 3))
+      {
+	// Strip one level of directory in the resource
+	filename += 3;
+
+	if ((slash = strrchr(resource, '/')) != NULL)
+	  *slash = '\0';
+      }
+
+      // Combine the resource and remaining relative filename to make a URL...
+      httpAssembleURIf(HTTP_URI_CODING_ALL, newfilename, sizeof(newfilename), scheme, userpass, host, port, "%s/%s", resource, filename);
     }
-    else if ((slash = strchr(base, '/')) == NULL)
-      strlcat(newfilename, "/", sizeof(newfilename));
   }
   else
   {
+    // Base is a filename...
     if (filename[0] == '/' || filename[0] == '\\' || base == NULL ||
 	base[0] == '\0' || (isalpha(filename[0]) && filename[1] == ':'))
-      return (file_find(Path, filename)); /* No change needed for absolute path */
+    {
+      // No change needed for absolute path...
+      return (file_find(Path, filename));
+    }
 
     strlcpy(newfilename, base, sizeof(newfilename));
     base = newfilename;
-  }
 
 #if defined(WIN32) || defined(__EMX__)
-  while (strncmp(filename, "../", 3) == 0 ||
-         strncmp(filename, "..\\", 3) == 0)
+    while (!strncmp(filename, "../", 3) || !strncmp(filename, "..\\", 3))
 #else
-  while (strncmp(filename, "../", 3) == 0)
+    while (!strncmp(filename, "../", 3))
 #endif // WIN32 || __EMX__
-  {
-    filename += 3;
-#if defined(WIN32) || defined(__EMX__)
-    if ((slash = strrchr(base, '/')) != NULL)
-      *slash = '\0';
-    else if ((slash = strrchr(base, '\\')) != NULL)
-      *slash = '\0';
-#else
-    if ((slash = strrchr(base, '/')) != NULL)
-      *slash = '\0';
-#endif // WIN32 || __EMX__
-    else
     {
-      filename -= 3;
-      break;
+      filename += 3;
+#if defined(WIN32) || defined(__EMX__)
+      if ((slash = strrchr(base, '/')) != NULL)
+        *slash = '\0';
+      else if ((slash = strrchr(base, '\\')) != NULL)
+        *slash = '\0';
+#else
+      if ((slash = strrchr(base, '/')) != NULL)
+        *slash = '\0';
+#endif // WIN32 || __EMX__
+      else
+      {
+	filename -= 3;
+	break;
+      }
     }
+
+    if (filename[0] != '/' && *base && base[strlen(base) - 1] != '/')
+      strlcat(newfilename, "/", sizeof(newfilename));
+
+    strlcat(newfilename, filename, sizeof(newfilename));
   }
 
-  if (filename[0] != '/' && *base && base[strlen(base) - 1] != '/')
-    strlcat(newfilename, "/", sizeof(newfilename));
-
-  strlcat(newfilename, filename, sizeof(newfilename));
-
 //  printf("    newfilename=\"%s\"\n", newfilename);
 
   return (file_find(Path, newfilename));
@@ -3721,7 +3778,6 @@ utf8_getc(int  ch,                      // I - Initial character
           FILE *fp)                     // I - File to read from
 {
   int  ch2 = -1, ch3 = -1;              // Temporary characters
-  int  unicode;                         // Unicode character
 
 
   if ((ch & 0xe0) == 0xc0)
@@ -3768,31 +3824,10 @@ utf8_getc(int  ch,                      // I - Initial character
     // them...  Try reading another character...
     //
     // TODO: Emit a warning about this...
-    if ((ch = utf8_getc(getc(fp), fp)) == 0)
-      return (0);
+    return (utf8_getc(getc(fp), fp));
   }
 
-  // If we already have a mapping for this character, return it...
-  if (_htmlCharacters[ch])
-    return (_htmlCharacters[ch]);
-
-  if (_htmlUTF8 >= 0x100)
-  {
-    progress_error(HD_ERROR_READ_ERROR, "Too many Unicode code points.");
-    return (0);
-  }
-
-  unicode = _htmlUTF8++;
-
-  _htmlCharacters[ch]   = (uchar)unicode;
-  _htmlUnicode[unicode] = ch;
-  _htmlGlyphs[unicode]  = _htmlGlyphsAll[ch];
-
-  for (int i = 0; i < TYPE_MAX; i ++)
-    for (int j = 0; j < STYLE_MAX; j ++)
-      _htmlWidths[i][j][unicode] = _htmlWidthsAll[i][j][ch];
-
-  return (unicode);
+  return (htmlMapUnicode(ch));
 
   bad_sequence:
 
